/**
 * @typedef {Object} MatchCaptureGroup
 * @property {string} text - text is matched texts
 * @property {number} index - index is start of match
 */
export declare type MatchCaptureGroup = {
    text: string;
    index: number;
};
export declare type MatchAllGroup = {
    all: string[];
    input: string;
    index: number;
    captureGroups: MatchCaptureGroup[];
};
/**
 * @param {string} text
 * @param {RegExp} regExp regExp should include capture
 * @returns {MatchCaptureGroup[]} return array of MatchCaptureGroup
 */
export declare function matchCaptureGroupAll(text: string, regExp: RegExp): MatchCaptureGroup[];
/**
 * matchAll function inspired String.prototype.matchAll
 * @param {String} text
 * @param {RegExp} regExp
 * @returns {MatchAllGroup[]}
 * @see reference https://github.com/tc39/String.prototype.matchAll
 * http://stackoverflow.com/questions/15934353/get-index-of-each-capture-in-a-javascript-regex
 */
export declare function matchAll(text: string, regExp: RegExp): MatchAllGroup[];
